/*
 * zrtp.org is a ZRTP protocol implementation  
 * Copyright (C) 2010 - PrivateWave Italia S.p.A.
 *  
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *  
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * For more information, please contact PrivateWave Italia S.p.A. at
 * address zorg@privatewave.com or http://www.privatewave.com
 */

#include <zorg/zorg.h>
#include <zorg/crypto.h>

const ::ZORG::Blob ZORG::Crypto::PGPWordList::EvenWords[256] =
{
    ::ZORG::asBlob("aardvark"),
    ::ZORG::asBlob("absurd"),
    ::ZORG::asBlob("accrue"),
    ::ZORG::asBlob("acme"),
    ::ZORG::asBlob("adrift"),
    ::ZORG::asBlob("adult"),
    ::ZORG::asBlob("afflict"),
    ::ZORG::asBlob("ahead"),
    ::ZORG::asBlob("aimless"),
    ::ZORG::asBlob("Algol"),
    ::ZORG::asBlob("allow"),
    ::ZORG::asBlob("alone"),
    ::ZORG::asBlob("ammo"),
    ::ZORG::asBlob("ancient"),
    ::ZORG::asBlob("apple"),
    ::ZORG::asBlob("artist"),
    ::ZORG::asBlob("assume"),
    ::ZORG::asBlob("Athens"),
    ::ZORG::asBlob("atlas"),
    ::ZORG::asBlob("Aztec"),
    ::ZORG::asBlob("baboon"),
    ::ZORG::asBlob("backfield"),
    ::ZORG::asBlob("backward"),
    ::ZORG::asBlob("banjo"),
    ::ZORG::asBlob("beaming"),
    ::ZORG::asBlob("bedlamp"),
    ::ZORG::asBlob("beehive"),
    ::ZORG::asBlob("beeswax"),
    ::ZORG::asBlob("befriend"),
    ::ZORG::asBlob("Belfast"),
    ::ZORG::asBlob("berserk"),
    ::ZORG::asBlob("billiard"),
    ::ZORG::asBlob("bison"),
    ::ZORG::asBlob("blackjack"),
    ::ZORG::asBlob("blockade"),
    ::ZORG::asBlob("blowtorch"),
    ::ZORG::asBlob("bluebird"),
    ::ZORG::asBlob("bombast"),
    ::ZORG::asBlob("bookshelf"),
    ::ZORG::asBlob("brackish"),
    ::ZORG::asBlob("breadline"),
    ::ZORG::asBlob("breakup"),
    ::ZORG::asBlob("brickyard"),
    ::ZORG::asBlob("briefcase"),
    ::ZORG::asBlob("Burbank"),
    ::ZORG::asBlob("button"),
    ::ZORG::asBlob("buzzard"),
    ::ZORG::asBlob("cement"),
    ::ZORG::asBlob("chairlift"),
    ::ZORG::asBlob("chatter"),
    ::ZORG::asBlob("checkup"),
    ::ZORG::asBlob("chisel"),
    ::ZORG::asBlob("choking"),
    ::ZORG::asBlob("chopper"),
    ::ZORG::asBlob("Christmas"),
    ::ZORG::asBlob("clamshell"),
    ::ZORG::asBlob("classic"),
    ::ZORG::asBlob("classroom"),
    ::ZORG::asBlob("cleanup"),
    ::ZORG::asBlob("clockwork"),
    ::ZORG::asBlob("cobra"),
    ::ZORG::asBlob("commence"),
    ::ZORG::asBlob("concert"),
    ::ZORG::asBlob("cowbell"),
    ::ZORG::asBlob("crackdown"),
    ::ZORG::asBlob("cranky"),
    ::ZORG::asBlob("crowfoot"),
    ::ZORG::asBlob("crucial"),
    ::ZORG::asBlob("crumpled"),
    ::ZORG::asBlob("crusade"),
    ::ZORG::asBlob("cubic"),
    ::ZORG::asBlob("dashboard"),
    ::ZORG::asBlob("deadbolt"),
    ::ZORG::asBlob("deckhand"),
    ::ZORG::asBlob("dogsled"),
    ::ZORG::asBlob("dragnet"),
    ::ZORG::asBlob("drainage"),
    ::ZORG::asBlob("dreadful"),
    ::ZORG::asBlob("drifter"),
    ::ZORG::asBlob("dropper"),
    ::ZORG::asBlob("drumbeat"),
    ::ZORG::asBlob("drunken"),
    ::ZORG::asBlob("Dupont"),
    ::ZORG::asBlob("dwelling"),
    ::ZORG::asBlob("eating"),
    ::ZORG::asBlob("edict"),
    ::ZORG::asBlob("egghead"),
    ::ZORG::asBlob("eightball"),
    ::ZORG::asBlob("endorse"),
    ::ZORG::asBlob("endow"),
    ::ZORG::asBlob("enlist"),
    ::ZORG::asBlob("erase"),
    ::ZORG::asBlob("escape"),
    ::ZORG::asBlob("exceed"),
    ::ZORG::asBlob("eyeglass"),
    ::ZORG::asBlob("eyetooth"),
    ::ZORG::asBlob("facial"),
    ::ZORG::asBlob("fallout"),
    ::ZORG::asBlob("flagpole"),
    ::ZORG::asBlob("flatfoot"),
    ::ZORG::asBlob("flytrap"),
    ::ZORG::asBlob("fracture"),
    ::ZORG::asBlob("framework"),
    ::ZORG::asBlob("freedom"),
    ::ZORG::asBlob("frighten"),
    ::ZORG::asBlob("gazelle"),
    ::ZORG::asBlob("Geiger"),
    ::ZORG::asBlob("glitter"),
    ::ZORG::asBlob("glucose"),
    ::ZORG::asBlob("goggles"),
    ::ZORG::asBlob("goldfish"),
    ::ZORG::asBlob("gremlin"),
    ::ZORG::asBlob("guidance"),
    ::ZORG::asBlob("hamlet"),
    ::ZORG::asBlob("highchair"),
    ::ZORG::asBlob("hockey"),
    ::ZORG::asBlob("indoors"),
    ::ZORG::asBlob("indulge"),
    ::ZORG::asBlob("inverse"),
    ::ZORG::asBlob("involve"),
    ::ZORG::asBlob("island"),
    ::ZORG::asBlob("jawbone"),
    ::ZORG::asBlob("keyboard"),
    ::ZORG::asBlob("kickoff"),
    ::ZORG::asBlob("kiwi"),
    ::ZORG::asBlob("klaxon"),
    ::ZORG::asBlob("locale"),
    ::ZORG::asBlob("lockup"),
    ::ZORG::asBlob("merit"),
    ::ZORG::asBlob("minnow"),
    ::ZORG::asBlob("miser"),
    ::ZORG::asBlob("Mohawk"),
    ::ZORG::asBlob("mural"),
    ::ZORG::asBlob("music"),
    ::ZORG::asBlob("necklace"),
    ::ZORG::asBlob("Neptune"),
    ::ZORG::asBlob("newborn"),
    ::ZORG::asBlob("nightbird"),
    ::ZORG::asBlob("Oakland"),
    ::ZORG::asBlob("obtuse"),
    ::ZORG::asBlob("offload"),
    ::ZORG::asBlob("optic"),
    ::ZORG::asBlob("orca"),
    ::ZORG::asBlob("payday"),
    ::ZORG::asBlob("peachy"),
    ::ZORG::asBlob("pheasant"),
    ::ZORG::asBlob("physique"),
    ::ZORG::asBlob("playhouse"),
    ::ZORG::asBlob("Pluto"),
    ::ZORG::asBlob("preclude"),
    ::ZORG::asBlob("prefer"),
    ::ZORG::asBlob("preshrunk"),
    ::ZORG::asBlob("printer"),
    ::ZORG::asBlob("prowler"),
    ::ZORG::asBlob("pupil"),
    ::ZORG::asBlob("puppy"),
    ::ZORG::asBlob("python"),
    ::ZORG::asBlob("quadrant"),
    ::ZORG::asBlob("quiver"),
    ::ZORG::asBlob("quota"),
    ::ZORG::asBlob("ragtime"),
    ::ZORG::asBlob("ratchet"),
    ::ZORG::asBlob("rebirth"),
    ::ZORG::asBlob("reform"),
    ::ZORG::asBlob("regain"),
    ::ZORG::asBlob("reindeer"),
    ::ZORG::asBlob("rematch"),
    ::ZORG::asBlob("repay"),
    ::ZORG::asBlob("retouch"),
    ::ZORG::asBlob("revenge"),
    ::ZORG::asBlob("reward"),
    ::ZORG::asBlob("rhythm"),
    ::ZORG::asBlob("ribcage"),
    ::ZORG::asBlob("ringbolt"),
    ::ZORG::asBlob("robust"),
    ::ZORG::asBlob("rocker"),
    ::ZORG::asBlob("ruffled"),
    ::ZORG::asBlob("sailboat"),
    ::ZORG::asBlob("sawdust"),
    ::ZORG::asBlob("scallion"),
    ::ZORG::asBlob("scenic"),
    ::ZORG::asBlob("scorecard"),
    ::ZORG::asBlob("Scotland"),
    ::ZORG::asBlob("seabird"),
    ::ZORG::asBlob("select"),
    ::ZORG::asBlob("sentence"),
    ::ZORG::asBlob("shadow"),
    ::ZORG::asBlob("shamrock"),
    ::ZORG::asBlob("showgirl"),
    ::ZORG::asBlob("skullcap"),
    ::ZORG::asBlob("skydive"),
    ::ZORG::asBlob("slingshot"),
    ::ZORG::asBlob("slowdown"),
    ::ZORG::asBlob("snapline"),
    ::ZORG::asBlob("snapshot"),
    ::ZORG::asBlob("snowcap"),
    ::ZORG::asBlob("snowslide"),
    ::ZORG::asBlob("solo"),
    ::ZORG::asBlob("southward"),
    ::ZORG::asBlob("soybean"),
    ::ZORG::asBlob("spaniel"),
    ::ZORG::asBlob("spearhead"),
    ::ZORG::asBlob("spellbind"),
    ::ZORG::asBlob("spheroid"),
    ::ZORG::asBlob("spigot"),
    ::ZORG::asBlob("spindle"),
    ::ZORG::asBlob("spyglass"),
    ::ZORG::asBlob("stagehand"),
    ::ZORG::asBlob("stagnate"),
    ::ZORG::asBlob("stairway"),
    ::ZORG::asBlob("standard"),
    ::ZORG::asBlob("stapler"),
    ::ZORG::asBlob("steamship"),
    ::ZORG::asBlob("sterling"),
    ::ZORG::asBlob("stockman"),
    ::ZORG::asBlob("stopwatch"),
    ::ZORG::asBlob("stormy"),
    ::ZORG::asBlob("sugar"),
    ::ZORG::asBlob("surmount"),
    ::ZORG::asBlob("suspense"),
    ::ZORG::asBlob("sweatband"),
    ::ZORG::asBlob("swelter"),
    ::ZORG::asBlob("tactics"),
    ::ZORG::asBlob("talon"),
    ::ZORG::asBlob("tapeworm"),
    ::ZORG::asBlob("tempest"),
    ::ZORG::asBlob("tiger"),
    ::ZORG::asBlob("tissue"),
    ::ZORG::asBlob("tonic"),
    ::ZORG::asBlob("topmost"),
    ::ZORG::asBlob("tracker"),
    ::ZORG::asBlob("transit"),
    ::ZORG::asBlob("trauma"),
    ::ZORG::asBlob("treadmill"),
    ::ZORG::asBlob("Trojan"),
    ::ZORG::asBlob("trouble"),
    ::ZORG::asBlob("tumor"),
    ::ZORG::asBlob("tunnel"),
    ::ZORG::asBlob("tycoon"),
    ::ZORG::asBlob("uncut"),
    ::ZORG::asBlob("unearth"),
    ::ZORG::asBlob("unwind"),
    ::ZORG::asBlob("uproot"),
    ::ZORG::asBlob("upset"),
    ::ZORG::asBlob("upshot"),
    ::ZORG::asBlob("vapor"),
    ::ZORG::asBlob("village"),
    ::ZORG::asBlob("virus"),
    ::ZORG::asBlob("Vulcan"),
    ::ZORG::asBlob("waffle"),
    ::ZORG::asBlob("wallet"),
    ::ZORG::asBlob("watchword"),
    ::ZORG::asBlob("wayside"),
    ::ZORG::asBlob("willow"),
    ::ZORG::asBlob("woodlark"),
    ::ZORG::asBlob("Zulu"),
};

const ::ZORG::Blob ZORG::Crypto::PGPWordList::OddWords[256] =
{
    ::ZORG::asBlob("adroitness"),
    ::ZORG::asBlob("adviser"),
    ::ZORG::asBlob("aftermath"),
    ::ZORG::asBlob("aggregate"),
    ::ZORG::asBlob("alkali"),
    ::ZORG::asBlob("almighty"),
    ::ZORG::asBlob("amulet"),
    ::ZORG::asBlob("amusement"),
    ::ZORG::asBlob("antenna"),
    ::ZORG::asBlob("applicant"),
    ::ZORG::asBlob("Apollo"),
    ::ZORG::asBlob("armistice"),
    ::ZORG::asBlob("article"),
    ::ZORG::asBlob("asteroid"),
    ::ZORG::asBlob("Atlantic"),
    ::ZORG::asBlob("atmosphere"),
    ::ZORG::asBlob("autopsy"),
    ::ZORG::asBlob("Babylon"),
    ::ZORG::asBlob("backwater"),
    ::ZORG::asBlob("barbecue"),
    ::ZORG::asBlob("belowground"),
    ::ZORG::asBlob("bifocals"),
    ::ZORG::asBlob("bodyguard"),
    ::ZORG::asBlob("bookseller"),
    ::ZORG::asBlob("borderline"),
    ::ZORG::asBlob("bottomless"),
    ::ZORG::asBlob("Bradbury"),
    ::ZORG::asBlob("bravado"),
    ::ZORG::asBlob("Brazilian"),
    ::ZORG::asBlob("breakaway"),
    ::ZORG::asBlob("Burlington"),
    ::ZORG::asBlob("businessman"),
    ::ZORG::asBlob("butterfat"),
    ::ZORG::asBlob("Camelot"),
    ::ZORG::asBlob("candidate"),
    ::ZORG::asBlob("cannonball"),
    ::ZORG::asBlob("Capricorn"),
    ::ZORG::asBlob("caravan"),
    ::ZORG::asBlob("caretaker"),
    ::ZORG::asBlob("celebrate"),
    ::ZORG::asBlob("cellulose"),
    ::ZORG::asBlob("certify"),
    ::ZORG::asBlob("chambermaid"),
    ::ZORG::asBlob("Cherokee"),
    ::ZORG::asBlob("Chicago"),
    ::ZORG::asBlob("clergyman"),
    ::ZORG::asBlob("coherence"),
    ::ZORG::asBlob("combustion"),
    ::ZORG::asBlob("commando"),
    ::ZORG::asBlob("company"),
    ::ZORG::asBlob("component"),
    ::ZORG::asBlob("concurrent"),
    ::ZORG::asBlob("confidence"),
    ::ZORG::asBlob("conformist"),
    ::ZORG::asBlob("congregate"),
    ::ZORG::asBlob("consensus"),
    ::ZORG::asBlob("consulting"),
    ::ZORG::asBlob("corporate"),
    ::ZORG::asBlob("corrosion"),
    ::ZORG::asBlob("councilman"),
    ::ZORG::asBlob("crossover"),
    ::ZORG::asBlob("crucifix"),
    ::ZORG::asBlob("cumbersome"),
    ::ZORG::asBlob("customer"),
    ::ZORG::asBlob("Dakota"),
    ::ZORG::asBlob("decadence"),
    ::ZORG::asBlob("December"),
    ::ZORG::asBlob("decimal"),
    ::ZORG::asBlob("designing"),
    ::ZORG::asBlob("detector"),
    ::ZORG::asBlob("detergent"),
    ::ZORG::asBlob("determine"),
    ::ZORG::asBlob("dictator"),
    ::ZORG::asBlob("dinosaur"),
    ::ZORG::asBlob("direction"),
    ::ZORG::asBlob("disable"),
    ::ZORG::asBlob("disbelief"),
    ::ZORG::asBlob("disruptive"),
    ::ZORG::asBlob("distortion"),
    ::ZORG::asBlob("document"),
    ::ZORG::asBlob("embezzle"),
    ::ZORG::asBlob("enchanting"),
    ::ZORG::asBlob("enrollment"),
    ::ZORG::asBlob("enterprise"),
    ::ZORG::asBlob("equation"),
    ::ZORG::asBlob("equipment"),
    ::ZORG::asBlob("escapade"),
    ::ZORG::asBlob("Eskimo"),
    ::ZORG::asBlob("everyday"),
    ::ZORG::asBlob("examine"),
    ::ZORG::asBlob("existence"),
    ::ZORG::asBlob("exodus"),
    ::ZORG::asBlob("fascinate"),
    ::ZORG::asBlob("filament"),
    ::ZORG::asBlob("finicky"),
    ::ZORG::asBlob("forever"),
    ::ZORG::asBlob("fortitude"),
    ::ZORG::asBlob("frequency"),
    ::ZORG::asBlob("gadgetry"),
    ::ZORG::asBlob("Galveston"),
    ::ZORG::asBlob("getaway"),
    ::ZORG::asBlob("glossary"),
    ::ZORG::asBlob("gossamer"),
    ::ZORG::asBlob("graduate"),
    ::ZORG::asBlob("gravity"),
    ::ZORG::asBlob("guitarist"),
    ::ZORG::asBlob("hamburger"),
    ::ZORG::asBlob("Hamilton"),
    ::ZORG::asBlob("handiwork"),
    ::ZORG::asBlob("hazardous"),
    ::ZORG::asBlob("headwaters"),
    ::ZORG::asBlob("hemisphere"),
    ::ZORG::asBlob("hesitate"),
    ::ZORG::asBlob("hideaway"),
    ::ZORG::asBlob("holiness"),
    ::ZORG::asBlob("hurricane"),
    ::ZORG::asBlob("hydraulic"),
    ::ZORG::asBlob("impartial"),
    ::ZORG::asBlob("impetus"),
    ::ZORG::asBlob("inception"),
    ::ZORG::asBlob("indigo"),
    ::ZORG::asBlob("inertia"),
    ::ZORG::asBlob("infancy"),
    ::ZORG::asBlob("inferno"),
    ::ZORG::asBlob("informant"),
    ::ZORG::asBlob("insincere"),
    ::ZORG::asBlob("insurgent"),
    ::ZORG::asBlob("integrate"),
    ::ZORG::asBlob("intention"),
    ::ZORG::asBlob("inventive"),
    ::ZORG::asBlob("Istanbul"),
    ::ZORG::asBlob("Jamaica"),
    ::ZORG::asBlob("Jupiter"),
    ::ZORG::asBlob("leprosy"),
    ::ZORG::asBlob("letterhead"),
    ::ZORG::asBlob("liberty"),
    ::ZORG::asBlob("maritime"),
    ::ZORG::asBlob("matchmaker"),
    ::ZORG::asBlob("maverick"),
    ::ZORG::asBlob("Medusa"),
    ::ZORG::asBlob("megaton"),
    ::ZORG::asBlob("microscope"),
    ::ZORG::asBlob("microwave"),
    ::ZORG::asBlob("midsummer"),
    ::ZORG::asBlob("millionaire"),
    ::ZORG::asBlob("miracle"),
    ::ZORG::asBlob("misnomer"),
    ::ZORG::asBlob("molasses"),
    ::ZORG::asBlob("molecule"),
    ::ZORG::asBlob("Montana"),
    ::ZORG::asBlob("monument"),
    ::ZORG::asBlob("mosquito"),
    ::ZORG::asBlob("narrative"),
    ::ZORG::asBlob("nebula"),
    ::ZORG::asBlob("newsletter"),
    ::ZORG::asBlob("Norwegian"),
    ::ZORG::asBlob("October"),
    ::ZORG::asBlob("Ohio"),
    ::ZORG::asBlob("onlooker"),
    ::ZORG::asBlob("opulent"),
    ::ZORG::asBlob("Orlando"),
    ::ZORG::asBlob("outfielder"),
    ::ZORG::asBlob("Pacific"),
    ::ZORG::asBlob("pandemic"),
    ::ZORG::asBlob("Pandora"),
    ::ZORG::asBlob("paperweight"),
    ::ZORG::asBlob("paragon"),
    ::ZORG::asBlob("paragraph"),
    ::ZORG::asBlob("paramount"),
    ::ZORG::asBlob("passenger"),
    ::ZORG::asBlob("pedigree"),
    ::ZORG::asBlob("Pegasus"),
    ::ZORG::asBlob("penetrate"),
    ::ZORG::asBlob("perceptive"),
    ::ZORG::asBlob("performance"),
    ::ZORG::asBlob("pharmacy"),
    ::ZORG::asBlob("phonetic"),
    ::ZORG::asBlob("photograph"),
    ::ZORG::asBlob("pioneer"),
    ::ZORG::asBlob("pocketful"),
    ::ZORG::asBlob("politeness"),
    ::ZORG::asBlob("positive"),
    ::ZORG::asBlob("potato"),
    ::ZORG::asBlob("processor"),
    ::ZORG::asBlob("provincial"),
    ::ZORG::asBlob("proximate"),
    ::ZORG::asBlob("puberty"),
    ::ZORG::asBlob("publisher"),
    ::ZORG::asBlob("pyramid"),
    ::ZORG::asBlob("quantity"),
    ::ZORG::asBlob("racketeer"),
    ::ZORG::asBlob("rebellion"),
    ::ZORG::asBlob("recipe"),
    ::ZORG::asBlob("recover"),
    ::ZORG::asBlob("repellent"),
    ::ZORG::asBlob("replica"),
    ::ZORG::asBlob("reproduce"),
    ::ZORG::asBlob("resistor"),
    ::ZORG::asBlob("responsive"),
    ::ZORG::asBlob("retraction"),
    ::ZORG::asBlob("retrieval"),
    ::ZORG::asBlob("retrospect"),
    ::ZORG::asBlob("revenue"),
    ::ZORG::asBlob("revival"),
    ::ZORG::asBlob("revolver"),
    ::ZORG::asBlob("sandalwood"),
    ::ZORG::asBlob("sardonic"),
    ::ZORG::asBlob("Saturday"),
    ::ZORG::asBlob("savagery"),
    ::ZORG::asBlob("scavenger"),
    ::ZORG::asBlob("sensation"),
    ::ZORG::asBlob("sociable"),
    ::ZORG::asBlob("souvenir"),
    ::ZORG::asBlob("specialist"),
    ::ZORG::asBlob("speculate"),
    ::ZORG::asBlob("stethoscope"),
    ::ZORG::asBlob("stupendous"),
    ::ZORG::asBlob("supportive"),
    ::ZORG::asBlob("surrender"),
    ::ZORG::asBlob("suspicious"),
    ::ZORG::asBlob("sympathy"),
    ::ZORG::asBlob("tambourine"),
    ::ZORG::asBlob("telephone"),
    ::ZORG::asBlob("therapist"),
    ::ZORG::asBlob("tobacco"),
    ::ZORG::asBlob("tolerance"),
    ::ZORG::asBlob("tomorrow"),
    ::ZORG::asBlob("torpedo"),
    ::ZORG::asBlob("tradition"),
    ::ZORG::asBlob("travesty"),
    ::ZORG::asBlob("trombonist"),
    ::ZORG::asBlob("truncated"),
    ::ZORG::asBlob("typewriter"),
    ::ZORG::asBlob("ultimate"),
    ::ZORG::asBlob("undaunted"),
    ::ZORG::asBlob("underfoot"),
    ::ZORG::asBlob("unicorn"),
    ::ZORG::asBlob("unify"),
    ::ZORG::asBlob("universe"),
    ::ZORG::asBlob("unravel"),
    ::ZORG::asBlob("upcoming"),
    ::ZORG::asBlob("vacancy"),
    ::ZORG::asBlob("vagabond"),
    ::ZORG::asBlob("vertigo"),
    ::ZORG::asBlob("Virginia"),
    ::ZORG::asBlob("visitor"),
    ::ZORG::asBlob("vocalist"),
    ::ZORG::asBlob("voyager"),
    ::ZORG::asBlob("warranty"),
    ::ZORG::asBlob("Waterloo"),
    ::ZORG::asBlob("whimsical"),
    ::ZORG::asBlob("Wichita"),
    ::ZORG::asBlob("Wilmington"),
    ::ZORG::asBlob("Wyoming"),
    ::ZORG::asBlob("yesteryear"),
    ::ZORG::asBlob("Yucatan")
};

// EOF
